/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizloadappoint.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.img.ImgUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fhs.trans.service.impl.TransService;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import org.apache.commons.compress.utils.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.auth.core.util.StpLoginUserUtil;
import vip.xiaonuo.biz.modular.bizconfig.entity.BizConfig;
import vip.xiaonuo.biz.modular.bizconfig.service.BizConfigService;
import vip.xiaonuo.biz.modular.bizloadappoint.param.*;
import vip.xiaonuo.biz.modular.bizloadappointsupplier.entity.BizLoadAppointSupplier;
import vip.xiaonuo.biz.modular.bizloadappointsupplier.service.BizLoadAppointSupplierService;
import vip.xiaonuo.biz.modular.bizorder.entity.BizOrder;
import vip.xiaonuo.biz.modular.bizorder.param.BizOrderExportResult;
import vip.xiaonuo.biz.modular.bizorderconfig.entity.BizOrderConfig;
import vip.xiaonuo.biz.modular.bizservicecustomer.entity.BizServiceCustomerAccount;
import vip.xiaonuo.biz.modular.bizservicecustomer.service.BizServiceCustomerAccountService;
import vip.xiaonuo.biz.modular.bizsettle.entity.BizSettle;
import vip.xiaonuo.biz.modular.utils.CommonExportUtil;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.biz.modular.bizloadappoint.entity.BizLoadAppoint;
import vip.xiaonuo.biz.modular.bizloadappoint.mapper.BizLoadAppointMapper;
import vip.xiaonuo.biz.modular.bizloadappoint.service.BizLoadAppointService;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.concurrent.locks.ReentrantLock;

/**
 * 起卸预约Service接口实现类
 *
 * @author fanzherong
 * @date  2025/06/25 14:51
 **/
@Service
public class BizLoadAppointServiceImpl extends ServiceImpl<BizLoadAppointMapper, BizLoadAppoint> implements BizLoadAppointService {

    private final ReentrantLock lock = new ReentrantLock();

    @Resource
    private BizLoadAppointSupplierService bizLoadAppointSupplierService;

    @Resource
    private TransService transService;

    @Resource
    private BizServiceCustomerAccountService bizServiceCustomerAccountService;

    @Resource
    private BizConfigService bizConfigService;

    @Override
    public Page<BizLoadAppoint> page(BizLoadAppointPageParam bizLoadAppointPageParam) {
        QueryWrapper<BizLoadAppoint> queryWrapper = getQueryWrapper(bizLoadAppointPageParam);
        Page<BizLoadAppoint> page = this.getBaseMapper().getPage(CommonPageRequest.defaultPage(), queryWrapper);
        for(BizLoadAppoint bizLoadAppoint : page.getRecords()){
            //查询汽运供应商
            List<String> idList = Lists.newArrayList();
            bizLoadAppointSupplierService.list(new QueryWrapper<BizLoadAppointSupplier>().lambda().
                    eq(BizLoadAppointSupplier::getAppointId,bizLoadAppoint.getId()))
                    .forEach(supplier->idList.add(supplier.getSupplierId()));
            bizLoadAppoint.setSupplierIdList(idList);
        }
        return page;
    }

    public QueryWrapper<BizLoadAppoint> getQueryWrapper(BizLoadAppointPageParam bizLoadAppointPageParam){
        QueryWrapper<BizLoadAppoint> queryWrapper = new QueryWrapper<BizLoadAppoint>().checkSqlInjection();
        //预约单号查询
        if(ObjectUtil.isNotEmpty(bizLoadAppointPageParam.getLoadNumber())){
            queryWrapper.like("bla.load_number",bizLoadAppointPageParam.getLoadNumber());
        }
        //主键id
        if(ObjectUtil.isNotEmpty(bizLoadAppointPageParam.getId())){
            queryWrapper.eq("bla.id",bizLoadAppointPageParam.getId());
        }
        //客户名称查询
        if(ObjectUtil.isNotEmpty(bizLoadAppointPageParam.getCustomerName())){
            queryWrapper.like("bsc.`name`",bizLoadAppointPageParam.getCustomerName());
        }
        //货品名称查询
        if(ObjectUtil.isNotEmpty(bizLoadAppointPageParam.getGoodsName())){
            queryWrapper.like("bg.GOODS_NAME",bizLoadAppointPageParam.getGoodsName());
        }
        //支付方式
        if(ObjectUtil.isNotEmpty(bizLoadAppointPageParam.getPayType())){
            queryWrapper.eq("bla.pay_type",bizLoadAppointPageParam.getPayType());
        }
        //创建时间
        if(ObjectUtil.isNotEmpty(bizLoadAppointPageParam.getCreateTimeBegin()) &&
                ObjectUtil.isNotEmpty(bizLoadAppointPageParam.getCreateTimeEnd())){
            queryWrapper.between("bla.create_time",bizLoadAppointPageParam.getCreateTimeBegin()+" 00:00:00",
                    bizLoadAppointPageParam.getCreateTimeEnd()+" 23:59:59");
        }
        // 校验数据范围
        List<String> loginUserDataScope = StpLoginUserUtil.getLoginUserDataScope();
        if(ObjectUtil.isEmpty(loginUserDataScope)) {
            queryWrapper.eq("bla.customer_id", StpLoginUserUtil.getLoginUser().getCustomerId());
        }
        // 起卸员
        if(StpLoginUserUtil.getLoginUser().getRoleCodeList().contains("load")){
            queryWrapper.eq("bld.user_id",StpLoginUserUtil.getLoginUser().getId());
        }
        queryWrapper.eq("bla.delete_flag","NOT_DELETE");
        queryWrapper.orderByDesc("bla.create_time");
        return queryWrapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(BizLoadAppointAddParam bizLoadAppointAddParam) {
        //校验
        checkParam(bizLoadAppointAddParam);
        //新增起卸预约信息
        BizLoadAppoint bizLoadAppoint = BeanUtil.toBean(bizLoadAppointAddParam, BizLoadAppoint.class);
        bizLoadAppoint.setLoadNumber(getNumber());
        bizLoadAppoint.setOrderWeight(bizLoadAppoint.getOrderWeight().multiply(new BigDecimal(1000)));
        this.save(bizLoadAppoint);

        if(StringUtils.equals(bizLoadAppointAddParam.getPayType(),"1")){
            //查询客户帐号信息
            BizServiceCustomerAccount customerAccount = bizServiceCustomerAccountService.getOne(new QueryWrapper<BizServiceCustomerAccount>().lambda().
                    eq(BizServiceCustomerAccount::getServiceCustomerId, bizLoadAppointAddParam.getCustomerId()).
                    last("limit 1"));
            //订单起卸费(起卸单价*订单数量)
            BigDecimal orderPrice = bizLoadAppointAddParam.getLoadPrice().multiply(bizLoadAppointAddParam.getOrderWeight());
            customerAccount.setLockAmount(customerAccount.getLockAmount().add(orderPrice));
            bizServiceCustomerAccountService.updateById(customerAccount);
        }


        //新增起卸预约汽运供应商
        if(ObjectUtil.isNotEmpty(bizLoadAppointAddParam.getSupplierIdList())){
            for(String supplierId : bizLoadAppointAddParam.getSupplierIdList()){
                BizLoadAppointSupplier bizLoadAppointSupplier = new BizLoadAppointSupplier();
                bizLoadAppointSupplier.setAppointId(bizLoadAppoint.getId());
                bizLoadAppointSupplier.setSupplierId(supplierId);
                bizLoadAppointSupplierService.save(bizLoadAppointSupplier);
            }
        }
    }

    public void checkParam(BizLoadAppointAddParam bizLoadAppointAddParam){
        lock.lock();
        try{
            if(StringUtils.equals(bizLoadAppointAddParam.getPayType(),"1")){
                //查询客户帐号信息
                BizServiceCustomerAccount customerAccount = bizServiceCustomerAccountService.getOne(new QueryWrapper<BizServiceCustomerAccount>().lambda().
                        eq(BizServiceCustomerAccount::getServiceCustomerId, bizLoadAppointAddParam.getCustomerId()).
                        last("limit 1"));
                if(ObjectUtil.isNull(customerAccount)){
                    throw new CommonException("未查询到客户信息！");
                }
                //可用金额 - 账户锁定金额
                BigDecimal amount = customerAccount.getAccountAmount().subtract(customerAccount.getLockAmount());
                if(amount.compareTo(BigDecimal.ZERO) <0){
                    throw new CommonException("客户账户可用余额不足！");
                }
                //订单起卸费(起卸单价*订单数量)
                BigDecimal orderPrice = bizLoadAppointAddParam.getLoadPrice().multiply(bizLoadAppointAddParam.getOrderWeight());
                if(amount.compareTo(orderPrice) <0){
                    throw new CommonException("客户账户余额不足以支付起卸费用！");
                }
            }
        }finally {
            //释放锁
            lock.unlock();
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(BizLoadAppointEditParam bizLoadAppointEditParam) {
        BizLoadAppoint bizLoadAppoint = this.queryEntity(bizLoadAppointEditParam.getId());
        checkParam(bizLoadAppointEditParam,bizLoadAppoint);
        BeanUtil.copyProperties(bizLoadAppointEditParam, bizLoadAppoint);
        //吨转kg
        bizLoadAppoint.setOrderWeight(bizLoadAppointEditParam.getOrderWeight().multiply(new BigDecimal(1000)));
        this.updateById(bizLoadAppoint);

        //删除原有汽运供应商
        List<BizLoadAppointSupplier> list = bizLoadAppointSupplierService.list(new QueryWrapper<BizLoadAppointSupplier>().lambda().
                eq(BizLoadAppointSupplier::getAppointId, bizLoadAppointEditParam.getId()));
        bizLoadAppointSupplierService.removeByIds(CollStreamUtil.toList(list,BizLoadAppointSupplier::getId));

        //新增汽运供应商
        if(ObjectUtil.isNotEmpty(bizLoadAppointEditParam.getSupplierIdList())){
            for(String supplierId : bizLoadAppointEditParam.getSupplierIdList()){
                BizLoadAppointSupplier bizLoadAppointSupplier = new BizLoadAppointSupplier();
                bizLoadAppointSupplier.setAppointId(bizLoadAppoint.getId());
                bizLoadAppointSupplier.setSupplierId(supplierId);
                bizLoadAppointSupplierService.save(bizLoadAppointSupplier);
            }
        }
    }

    public void checkParam(BizLoadAppointEditParam bizLoadAppointEditParam,BizLoadAppoint bizLoadAppoint){
        if(!StringUtils.equals(bizLoadAppointEditParam.getCustomerId(),bizLoadAppoint.getCustomerId())){
            lock.lock();
            try{
                if(StringUtils.equals(bizLoadAppointEditParam.getPayType(),"1")){
                    //查询客户帐号信息
                    BizServiceCustomerAccount customerAccount = bizServiceCustomerAccountService.getOne(new QueryWrapper<BizServiceCustomerAccount>().lambda().
                            eq(BizServiceCustomerAccount::getServiceCustomerId, bizLoadAppointEditParam.getCustomerId()).
                            last("limit 1"));
                    if(ObjectUtil.isNull(customerAccount)){
                        throw new CommonException("未查询到客户信息！");
                    }
                    //可用金额 - 账户锁定金额
                    BigDecimal amount = customerAccount.getAccountAmount().subtract(customerAccount.getLockAmount());
                    if(amount.compareTo(BigDecimal.ZERO) < 0){
                        throw new CommonException("客户账户可用余额不足！");
                    }
                    //订单起卸费(起卸单价*订单数量)
                    BigDecimal orderPrice = bizLoadAppointEditParam.getLoadPrice().multiply(bizLoadAppointEditParam.getOrderWeight());
                    if(amount.compareTo(orderPrice) < 0){
                        throw new CommonException("客户账户余额不足以支付起卸费用！");
                    }


                    //查询旧客户信息
                    //查询客户帐号信息
                    BizServiceCustomerAccount oldCustomerAccount = bizServiceCustomerAccountService.getOne(new QueryWrapper<BizServiceCustomerAccount>().lambda().
                            eq(BizServiceCustomerAccount::getServiceCustomerId, bizLoadAppoint.getCustomerId()).
                            last("limit 1"));
                    //订单起卸费(起卸单价*订单数量)
                    BigDecimal olderPrice = bizLoadAppoint.getLoadPrice().divide(new BigDecimal(1000)).multiply(bizLoadAppoint.getOrderWeight());
                    oldCustomerAccount.setLockAmount(oldCustomerAccount.getLockAmount().subtract(olderPrice));
                    bizServiceCustomerAccountService.updateById(oldCustomerAccount);

                    //查询新客户信息
                    BizServiceCustomerAccount newCustomerAccount = bizServiceCustomerAccountService.getOne(new QueryWrapper<BizServiceCustomerAccount>().lambda().
                            eq(BizServiceCustomerAccount::getServiceCustomerId, bizLoadAppointEditParam.getCustomerId()).
                            last("limit 1"));
                    //订单起卸费(起卸单价*订单数量)
                    BigDecimal newPrice = bizLoadAppointEditParam.getLoadPrice().multiply(bizLoadAppointEditParam.getOrderWeight());
                    newCustomerAccount.setLockAmount(newCustomerAccount.getLockAmount().add(newPrice));
                    bizServiceCustomerAccountService.updateById(newCustomerAccount);
                }
            }finally {
                //释放锁
                lock.unlock();
            }
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<BizLoadAppointIdParam> bizLoadAppointIdParamList) {
        // 执行删除
        //删除供应商
        for(BizLoadAppointIdParam bizLoadAppointIdParam : bizLoadAppointIdParamList){
            List<BizLoadAppointSupplier> list = bizLoadAppointSupplierService.list(new QueryWrapper<BizLoadAppointSupplier>().lambda().
                    eq(BizLoadAppointSupplier::getAppointId, bizLoadAppointIdParam.getId()));
            bizLoadAppointSupplierService.removeByIds(CollStreamUtil.toList(list,BizLoadAppointSupplier::getId));

            BizLoadAppoint bizLoadAppoint = this.queryEntity(bizLoadAppointIdParam.getId());
            BizServiceCustomerAccount oldCustomerAccount = bizServiceCustomerAccountService.getOne(new QueryWrapper<BizServiceCustomerAccount>().lambda().
                    eq(BizServiceCustomerAccount::getServiceCustomerId, bizLoadAppoint.getCustomerId()).
                    last("limit 1"));
            if(ObjectUtil.isNotNull(oldCustomerAccount)){
                //订单起卸费(起卸单价*订单数量)
                BigDecimal olderPrice = bizLoadAppoint.getLoadPrice().divide(new BigDecimal(1000)).multiply(bizLoadAppoint.getOrderWeight());
                oldCustomerAccount.setLockAmount(oldCustomerAccount.getLockAmount().subtract(olderPrice));
                bizServiceCustomerAccountService.updateById(oldCustomerAccount);
            }


            this.removeById(bizLoadAppointIdParam.getId());
        }
        //this.removeByIds(CollStreamUtil.toList(bizLoadAppointIdParamList, BizLoadAppointIdParam::getId));
    }

    @Override
    public BizLoadAppoint detail(BizLoadAppointIdParam bizLoadAppointIdParam) {
        QueryWrapper<BizLoadAppoint> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isNotEmpty(bizLoadAppointIdParam.getId())){
            queryWrapper.eq("bla.id",bizLoadAppointIdParam.getId());
        }
        BizLoadAppoint detail = this.getBaseMapper().getDetail(queryWrapper);
        return detail;
    }

    @Override
    public BizLoadAppoint queryEntity(String id) {
        BizLoadAppoint bizLoadAppoint = this.getById(id);
        if(ObjectUtil.isEmpty(bizLoadAppoint)) {
            throw new CommonException("起卸预约不存在，id值为：{}", id);
        }
        return bizLoadAppoint;
    }

    @Override
    public void submit(BizLoadAppointEditParam bizLoadAppointEditParam) {
        BizLoadAppoint bizLoadAppoint = this.queryEntity(bizLoadAppointEditParam.getId());
        bizLoadAppoint.setStatus("1");

        //存取预约次数配置
        BizConfig bizConfig = bizConfigService.getOne(new QueryWrapper<BizConfig>().lambda().last("limit 1"));
        BigDecimal applyCount = bizLoadAppoint.getOrderWeight().divide(bizConfig.getLoadWeight(), RoundingMode.FLOOR);
        bizLoadAppoint.setAppointNumber(applyCount.intValue());
        this.updateById(bizLoadAppoint);
    }

    @Override
    public void updateCustomerSign(BizLoadAppointEditParam bizLoadAppointEditParam) {
        BizLoadAppoint bizLoadAppoint = this.queryEntity(bizLoadAppointEditParam.getId());
        if(ObjectUtil.isEmpty(bizLoadAppointEditParam.getCustomerSign())){
            throw new CommonException("签名不能为空！");
        }
        String customerSign = bizLoadAppointEditParam.getCustomerSign();
        if(bizLoadAppointEditParam.getCustomerSign().contains(StrUtil.COMMA)) {
            customerSign = StrUtil.split(customerSign, StrUtil.COMMA).get(1);
        }
        String base64 = ImgUtil.toBase64DataUri(ImgUtil.scale(ImgUtil.toImage(customerSign),
                100, 50, null), ImgUtil.IMAGE_TYPE_PNG);

        bizLoadAppoint.setCustomerSign(base64);
        bizLoadAppoint.setStatus("2");
        bizLoadAppoint.setSignTime(new Date());
        this.updateById(bizLoadAppoint);
    }

    @Override
    public void exportRecord(BizLoadAppointPageParam bizLoadAppointPageParam, HttpServletResponse response) {
        QueryWrapper<BizLoadAppoint> queryWrapper = getQueryWrapper(bizLoadAppointPageParam);
        List<BizLoadAppoint> appointList = this.getBaseMapper().getPage(queryWrapper);
        if(ObjectUtil.isEmpty(appointList)){
            throw new CommonException("无数据可导出！");
        }
        String fileName = "起卸预约报表.xlsx";
        List<BizLoadAppointExportResult> list = Lists.newArrayList();
        transService.transBatch(appointList);
        for(BizLoadAppoint bizLoadAppoint : appointList){
            BizLoadAppointExportResult bizLoadAppointExportResult = new BizLoadAppointExportResult();
            BeanUtil.copyProperties(bizLoadAppoint, bizLoadAppointExportResult);
            bizLoadAppointExportResult.setStatus((String) bizLoadAppoint.getTransMap().get("statusName"));
            list.add(bizLoadAppointExportResult);
        }
        CommonExportUtil.export(fileName, BizLoadAppointExportResult.class,list,response,"起卸预约报表");
    }

    @Override
    public List<BizLoadAppoint> getList(BizLoadAppointPageParam bizLoadAppointPageParam) {
        QueryWrapper<BizLoadAppoint> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isNotEmpty(bizLoadAppointPageParam.getStatus())){
            List<String> statusList = Arrays.asList(bizLoadAppointPageParam.getStatus().split(","));
            queryWrapper.lambda().in(BizLoadAppoint::getStatus,statusList);
        }
        List<BizLoadAppoint> list = this.list(queryWrapper);
        return list;
    }

    @Override
    public void endLoad(BizLoadAppointEditParam bizLoadAppointEditParam) {
        BizLoadAppoint bizLoadAppoint = this.queryEntity(bizLoadAppointEditParam.getId());
        bizLoadAppoint.setEndReason(bizLoadAppointEditParam.getEndReason());
        bizLoadAppoint.setStatus("7");
        this.updateById(bizLoadAppoint);
    }

    @Override
    public void fillLoad(BizLoadAppointEditParam bizLoadAppointEditParam) {
        BizLoadAppoint bizLoadAppoint = this.queryEntity(bizLoadAppointEditParam.getId());
        bizLoadAppoint.setStatus("8");
        bizLoadAppoint.setFillWeight(bizLoadAppointEditParam.getFillWeight().multiply(new BigDecimal(1000)));
        bizLoadAppoint.setFillTime(new Date());
        this.updateById(bizLoadAppoint);
    }

    @Override
    public void updateAppointNumber(BizLoadAppointEditParam bizLoadAppointEditParam) {
        BizLoadAppoint bizLoadAppoint = this.queryEntity(bizLoadAppointEditParam.getId());
        if(bizLoadAppointEditParam.getAppointNumber() <= bizLoadAppoint.getAlreadyAppointNumber()){
            throw new CommonException("申请数量不能小于等于已约数量！");
        }
        bizLoadAppoint.setAppointNumber(bizLoadAppointEditParam.getAppointNumber());
        this.updateById(bizLoadAppoint);
    }


    public String getNumber(){
        lock.lock();
        try{
            String format = new SimpleDateFormat("yyyyMMdd").format(new Date());
            String time = new SimpleDateFormat("yyyy-MM-dd").format(new Date());
            long count = this.count(new QueryWrapper<BizLoadAppoint>().lambda().between(BizLoadAppoint::getCreateTime, time + " 00:00:00", time + " 23:59:59"));
            String settleNo = "QX" + format + generateCode((int)count);
            return settleNo;
        }finally {
            lock.unlock(); // 释放锁
        }

    }

    public static String generateCode(Integer counter) {
        counter++;
        return String.format("%03d", counter);
    }
}
